@@ -7,7 +7,11 @@ ndk_paths = {
     'r14b': "/home/vagrant/android-ndk/r14b",
     'r15c': "/home/vagrant/android-ndk/r15c",
     'r16b': "/home/vagrant/android-ndk/r16b",
+    'r17b': "/home/vagrant/android-ndk/r17b",
+    'r18b': "/home/vagrant/android-ndk/r18b",
+    'r19': "/home/vagrant/android-ndk/r19",
 }
 java_paths = {
     '8': "/usr/lib/jvm/java-8-openjdk-amd64",
 }
+gradle_version_dir = "/opt/gradle/versions"
