@@ -21,6 +21,7 @@ import os
 import shutil
 import glob
 import subprocess
+import posixpath
 import re
 import resource
 import sys
@@ -30,8 +31,8 @@ import traceback
 import time
 import requests
 import tempfile
+import argparse
 from configparser import ConfigParser
-from argparse import ArgumentParser
 import logging
 from gettext import ngettext
 
@@ -41,7 +42,7 @@ from . import net
 from . import metadata
 from . import scanner
 from . import vmtools
-from .common import FDroidPopen, SdkToolsPopen
+from .common import FDroidPopen
 from .exception import FDroidException, BuildException, VCSException
 
 try:
@@ -66,8 +67,8 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
 
     try:
         paramiko
-    except NameError:
-        raise BuildException("Paramiko is required to use the buildserver")
+    except NameError as e:
+        raise BuildException("Paramiko is required to use the buildserver") from e
     if options.verbose:
         logging.getLogger("paramiko").setLevel(logging.INFO)
     else:
@@ -75,13 +76,21 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
 
     sshinfo = vmtools.get_clean_builder('builder', options.reset_server)
 
+    output = None
     try:
         if not buildserverid:
-            buildserverid = subprocess.check_output(['vagrant', 'ssh', '-c',
-                                                     'cat /home/vagrant/buildserverid'],
-                                                    cwd='builder').strip().decode()
-            logging.debug(_('Fetched buildserverid from VM: {buildserverid}')
-                          .format(buildserverid=buildserverid))
+            try:
+                buildserverid = subprocess.check_output(['vagrant', 'ssh', '-c',
+                                                         'cat /home/vagrant/buildserverid'],
+                                                        cwd='builder').strip().decode()
+                logging.debug(_('Fetched buildserverid from VM: {buildserverid}')
+                              .format(buildserverid=buildserverid))
+            except Exception as e:
+                if type(buildserverid) is not str or not re.match('^[0-9a-f]{40}$', buildserverid):
+                    logging.info(subprocess.check_output(['vagrant', 'status'], cwd="builder"))
+                    raise FDroidException("Could not obtain buildserverid from buldserver VM. "
+                                          "(stored inside the buildserver VM at '/home/vagrant/buildserverid') "
+                                          "Please reset your buildserver, the setup VM is broken.") from e
 
         # Open SSH connection...
         logging.info("Connecting to virtual machine...")
@@ -91,7 +100,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
                      port=sshinfo['port'], timeout=300,
                      look_for_keys=False, key_filename=sshinfo['idfile'])
 
-        homedir = '/home/' + sshinfo['user']
+        homedir = posixpath.join('/home', sshinfo['user'])
 
         # Get an SFTP connection...
         ftp = sshs.open_sftp()
@@ -124,7 +133,9 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         ftp.mkdir('fdroidserver')
         ftp.chdir('fdroidserver')
         ftp.put(os.path.join(serverpath, '..', 'fdroid'), 'fdroid')
-        ftp.chmod('fdroid', 0o755)
+        ftp.put(os.path.join(serverpath, '..', 'gradlew-fdroid'), 'gradlew-fdroid')
+        ftp.chmod('fdroid', 0o755)  # nosec B103 permissions are appropriate
+        ftp.chmod('gradlew-fdroid', 0o755)  # nosec B103 permissions are appropriate
         send_dir(os.path.join(serverpath))
         ftp.chdir(homedir)
 
@@ -156,7 +167,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         ftp.mkdir('srclib')
         # Copy any extlibs that are required...
         if build.extlibs:
-            ftp.chdir(homedir + '/build/extlib')
+            ftp.chdir(posixpath.join(homedir, 'build', 'extlib'))
             for lib in build.extlibs:
                 lib = lib.strip()
                 libsrc = os.path.join('build/extlib', lib)
@@ -183,20 +194,20 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             srclibpaths.append(basesrclib)
         for name, number, lib in srclibpaths:
             logging.info("Sending srclib '%s'" % lib)
-            ftp.chdir(homedir + '/build/srclib')
+            ftp.chdir(posixpath.join(homedir, 'build', 'srclib'))
             if not os.path.exists(lib):
                 raise BuildException("Missing srclib directory '" + lib + "'")
             fv = '.fdroidvcs-' + name
             ftp.put(os.path.join('build/srclib', fv), fv)
             send_dir(lib)
             # Copy the metadata file too...
-            ftp.chdir(homedir + '/srclibs')
+            ftp.chdir(posixpath.join(homedir, 'srclibs'))
             ftp.put(os.path.join('srclibs', name + '.txt'),
                     name + '.txt')
         # Copy the main app source code
         # (no need if it's a srclib)
         if (not basesrclib) and os.path.exists(build_dir):
-            ftp.chdir(homedir + '/build')
+            ftp.chdir(posixpath.join(homedir, 'build'))
             fv = '.fdroidvcs-' + app.id
             ftp.put(os.path.join('build', fv), fv)
             send_dir(build_dir)
@@ -205,7 +216,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         logging.info("Starting build...")
         chan = sshs.get_transport().open_session()
         chan.get_pty()
-        cmdline = os.path.join(homedir, 'fdroidserver', 'fdroid')
+        cmdline = posixpath.join(homedir, 'fdroidserver', 'fdroid')
         cmdline += ' build --on-server'
         if force:
             cmdline += ' --force --test'
@@ -213,8 +224,10 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             cmdline += ' --verbose'
         if options.skipscan:
             cmdline += ' --skip-scan'
+        if options.notarball:
+            cmdline += ' --no-tarball'
         cmdline += " %s:%s" % (app.id, build.versionCode)
-        chan.exec_command('bash --login -c "' + cmdline + '"')
+        chan.exec_command('bash --login -c "' + cmdline + '"')  # nosec B601 inputs are sanitized
 
         # Fetch build process output ...
         try:
@@ -250,7 +263,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         # Retreive logs...
         toolsversion_log = common.get_toolsversion_logname(app, build)
         try:
-            ftp.chdir(os.path.join(homedir, log_dir))
+            ftp.chdir(posixpath.join(homedir, log_dir))
             ftp.get(toolsversion_log, os.path.join(log_dir, toolsversion_log))
             logging.debug('retrieved %s', toolsversion_log)
         except Exception as e:
@@ -259,9 +272,9 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         # Retrieve the built files...
         logging.info("Retrieving build output...")
         if force:
-            ftp.chdir(homedir + '/tmp')
+            ftp.chdir(posixpath.join(homedir, 'tmp'))
         else:
-            ftp.chdir(homedir + '/unsigned')
+            ftp.chdir(posixpath.join(homedir, 'unsigned'))
         apkfile = common.get_release_filename(app, build)
         tarball = common.getsrcname(app, build)
         try:
@@ -279,6 +292,13 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         vm = vmtools.get_build_vm('builder')
         vm.suspend()
 
+        # deploy logfile to repository web server
+        if output:
+            common.deploy_build_log_with_rsync(app.id, build.versionCode, output)
+        else:
+            logging.debug('skip publishing full build logs: '
+                          'no output present')
+
 
 def force_gradle_build_tools(build_dir, build_tools):
     for root, dirs, files in os.walk(build_dir):
@@ -303,92 +323,28 @@ def transform_first_char(string, method):
     return method(string[0]) + string[1:]
 
 
-def has_native_code(apkobj):
-    """aapt checks if there are architecture folders under the lib/ folder
-    so we are simulating the same behaviour"""
-    arch_re = re.compile("^lib/(.*)/.*$")
-    arch = [file for file in apkobj.get_files() if arch_re.match(file)]
-    return False if not arch else True
-
-
-def get_apk_metadata_aapt(apkfile):
-    """aapt function to extract versionCode, versionName, packageName and nativecode"""
-    vercode = None
-    version = None
-    foundid = None
-    nativecode = None
-
-    p = SdkToolsPopen(['aapt', 'dump', 'badging', apkfile], output=False)
-
-    for line in p.output.splitlines():
-        if line.startswith("package:"):
-            pat = re.compile(".*name='([a-zA-Z0-9._]*)'.*")
-            m = pat.match(line)
-            if m:
-                foundid = m.group(1)
-            pat = re.compile(".*versionCode='([0-9]*)'.*")
-            m = pat.match(line)
-            if m:
-                vercode = m.group(1)
-            pat = re.compile(".*versionName='([^']*)'.*")
-            m = pat.match(line)
-            if m:
-                version = m.group(1)
-        elif line.startswith("native-code:"):
-            nativecode = line[12:]
-
-    return vercode, version, foundid, nativecode
-
-
-def get_apk_metadata_androguard(apkfile):
-    """androguard function to extract versionCode, versionName, packageName and nativecode"""
-    try:
-        from androguard.core.bytecodes.apk import APK
-        apkobject = APK(apkfile)
-    except ImportError:
-        raise BuildException("androguard library is not installed and aapt binary not found")
-    except FileNotFoundError:
-        raise BuildException("Could not open apk file for metadata analysis")
-
-    if not apkobject.is_valid_APK():
-        raise BuildException("Invalid APK provided")
-
-    foundid = apkobject.get_package()
-    vercode = apkobject.get_androidversion_code()
-    version = apkobject.get_androidversion_name()
-    nativecode = has_native_code(apkobject)
-
-    return vercode, version, foundid, nativecode
-
-
 def get_metadata_from_apk(app, build, apkfile):
-    """get the required metadata from the built APK"""
+    """get the required metadata from the built APK
 
-    if common.SdkToolsPopen(['aapt', 'version'], output=False):
-        vercode, version, foundid, nativecode = get_apk_metadata_aapt(apkfile)
-    else:
-        vercode, version, foundid, nativecode = get_apk_metadata_androguard(apkfile)
+    versionName is allowed to be a blank string, i.e. ''
+    """
 
-    # Ignore empty strings or any kind of space/newline chars that we don't
-    # care about
-    if nativecode is not None:
-        nativecode = nativecode.strip()
-        nativecode = None if not nativecode else nativecode
+    appid, versionCode, versionName = common.get_apk_id(apkfile)
+    native_code = common.get_native_code(apkfile)
 
-    if build.buildjni and build.buildjni != ['no']:
-        if nativecode is None:
-            raise BuildException("Native code should have been built but none was packaged")
+    if build.buildjni and build.buildjni != ['no'] and not native_code:
+        raise BuildException("Native code should have been built but none was packaged")
     if build.novcheck:
-        vercode = build.versionCode
-        version = build.versionName
-    if not version or not vercode:
+        versionCode = build.versionCode
+        versionName = build.versionName
+    if not versionCode or versionName is None:
         raise BuildException("Could not find version information in build in output")
-    if not foundid:
+    if not appid:
         raise BuildException("Could not find package ID in output")
-    if foundid != app.id:
-        raise BuildException("Wrong package ID - build " + foundid + " but expected " + app.id)
+    if appid != app.id:
+        raise BuildException("Wrong package ID - build " + appid + " but expected " + app.id)
 
-    return vercode, version
+    return versionCode, versionName
 
 
 def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, extlib_dir, tmp_dir, force, onserver, refresh):
@@ -486,8 +442,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
             cmd += ['-P' + kv for kv in build.gradleprops]
 
         cmd += ['clean']
-
-        p = FDroidPopen(cmd, cwd=root_dir)
+        p = FDroidPopen(cmd, cwd=root_dir, envs={"GRADLE_VERSION_DIR": config['gradle_version_dir'], "CACHEDIR": config['cachedir']})
 
     elif bmethod == 'buildozer':
         pass
@@ -557,9 +512,9 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
         tarname = common.getsrcname(app, build)
         tarball = tarfile.open(os.path.join(tmp_dir, tarname), "w:gz")
 
-        def tarexc(f):
-            return any(f.endswith(s) for s in ['.svn', '.git', '.hg', '.bzr'])
-        tarball.add(build_dir, tarname, exclude=tarexc)
+        def tarexc(t):
+            return None if any(t.name.endswith(s) for s in ['.svn', '.git', '.hg', '.bzr']) else t
+        tarball.add(build_dir, tarname, filter=tarexc)
         tarball.close()
 
     # Run a build command if one is required...
@@ -710,7 +665,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
 
         cmd += gradletasks
 
-        p = FDroidPopen(cmd, cwd=root_dir)
+        p = FDroidPopen(cmd, cwd=root_dir, envs={"GRADLE_VERSION_DIR": config['gradle_version_dir'], "CACHEDIR": config['cachedir']})
 
     elif bmethod == 'ant':
         logging.info("Building Ant project...")
@@ -908,7 +863,7 @@ def force_halt_build(timeout):
 def parse_commandline():
     """Parse the command line. Returns options, parser."""
 
-    parser = ArgumentParser(usage="%(prog)s [options] [APPID[:VERCODE] [APPID[:VERCODE] ...]]")
+    parser = argparse.ArgumentParser(usage="%(prog)s [options] [APPID[:VERCODE] [APPID[:VERCODE] ...]]")
     common.setup_global_opts(parser)
     parser.add_argument("appid", nargs='*', help=_("applicationId with optional versionCode in the form APPID[:VERCODE]"))
     parser.add_argument("-l", "--latest", action="store_true", default=False,
@@ -921,8 +876,10 @@ def parse_commandline():
                         help=_("Use build server"))
     parser.add_argument("--reset-server", action="store_true", default=False,
                         help=_("Reset and create a brand new build server, even if the existing one appears to be ok."))
+    # this option is internal API for telling fdroid that
+    # it's running inside a buildserver vm.
     parser.add_argument("--on-server", dest="onserver", action="store_true", default=False,
-                        help=_("Specify that we're running on the build server"))
+                        help=argparse.SUPPRESS)
     parser.add_argument("--skip-scan", dest="skipscan", action="store_true", default=False,
                         help=_("Skip scanning the source code for binaries and other problems"))
     parser.add_argument("--dscanner", action="store_true", default=False,
@@ -1015,6 +972,7 @@ def main():
         if not os.path.isdir(output_dir):
             logging.info("Creating output directory")
             os.makedirs(output_dir)
+    binaries_dir = os.path.join(output_dir, 'binaries')
 
     if config['archive_older'] != 0:
         also_check_dir = 'archive'
@@ -1131,19 +1089,25 @@ def main():
                         # binary. We get that binary now, and save it
                         # alongside our built one in the 'unsigend'
                         # directory.
+                        if not os.path.isdir(binaries_dir):
+                            os.makedirs(binaries_dir)
+                            logging.info("Created directory for storing "
+                                         "developer supplied reference "
+                                         "binaries: '{path}'"
+                                         .format(path=binaries_dir))
                         url = app.Binaries
                         url = url.replace('%v', build.versionName)
                         url = url.replace('%c', str(build.versionCode))
                         logging.info("...retrieving " + url)
                         of = re.sub(r'.apk$', '.binary.apk', common.get_release_filename(app, build))
-                        of = os.path.join(output_dir, of)
+                        of = os.path.join(binaries_dir, of)
                         try:
                             net.download_file(url, local_filename=of)
                         except requests.exceptions.HTTPError as e:
                             raise FDroidException(
-                                'Downloading Binaries from %s failed. %s' % (url, e))
+                                'Downloading Binaries from %s failed.' % url) from e
 
-                        # Now we check weather the build can be verified to
+                        # Now we check whether the build can be verified to
                         # match the supplied binary or not. Should the
                         # comparison fail, we mark this build as a failure
                         # and remove everything from the unsigend folder.
@@ -1186,7 +1150,7 @@ def main():
                     appid, reason))
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
-                    sys.exit(1)
+                    common.force_exit(1)
                 failed_apps[appid] = vcse
                 wikilog = str(vcse)
             except FDroidException as e:
@@ -1201,7 +1165,7 @@ def main():
                 logging.error("Could not build app %s: %s" % (appid, e))
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
-                    sys.exit(1)
+                    common.force_exit(1)
                 failed_apps[appid] = e
                 wikilog = e.get_wikitext()
             except Exception as e:
@@ -1209,7 +1173,7 @@ def main():
                     appid, traceback.format_exc()))
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
-                    sys.exit(1)
+                    common.force_exit(1)
                 failed_apps[appid] = e
                 wikilog = str(e)
 
@@ -1276,7 +1240,7 @@ def main():
 
                 if not os.path.isdir(repo_dir):
                     logging.critical("directory does not exists '{path}'".format(path=repo_dir))
-                    sys.exit(1)
+                    common.force_exit(1)
 
                 logging.info("Performing Drozer scan on {0}.".format(app))
                 docker.perform_drozer_scan(apk_path, app.id, repo_dir)
@@ -1327,9 +1291,7 @@ def main():
         newpage.save('#REDIRECT [[' + wiki_page_path + ']]', summary='Update redirect')
 
     # hack to ensure this exits, even is some threads are still running
-    sys.stdout.flush()
-    sys.stderr.flush()
-    os._exit(0)
+    common.force_exit()
 
 
 if __name__ == "__main__":
