@@ -227,11 +227,11 @@ def main():
     if local_metadata_files != []:
         raise FDroidException(_("This repo already has local metadata: %s") % local_metadata_files[0])
 
+    build = metadata.Build()
     if options.url is None and os.path.isdir('.git'):
         app.AutoName = os.path.basename(os.getcwd())
         app.RepoType = 'git'
 
-        build = metadata.Build()
         root_dir = get_subdir(os.getcwd())
         if os.path.exists('build.gradle'):
             build.gradle = ['yes']
@@ -250,7 +250,6 @@ def main():
         write_local_file = True
     elif options.url:
         root_dir, build_dir = get_metadata_from_url(app, options.url)
-        build = metadata.Build()
         build.commit = '?'
         build.disable = 'Generated by import.py - check/set version fields and commit id'
         write_local_file = False
@@ -286,14 +285,14 @@ def main():
         raise FDroidException("Package " + package + " already exists")
 
     # Create a build line...
-    build.versionName = versionName or '?'
-    build.versionCode = versionCode or '?'
+    build.versionName = versionName or 'Unknown'
+    build.versionCode = versionCode or '0'  # TODO heinous but this is still a str
     if options.subdir:
         build.subdir = options.subdir
     if options.license:
         app.License = options.license
     if options.categories:
-        app.Categories = options.categories
+        app.Categories = options.categories.split(',')
     if os.path.exists(os.path.join(root_dir, 'jni')):
         build.buildjni = ['yes']
     if os.path.exists(os.path.join(root_dir, 'build.gradle')):
