@@ -165,6 +165,8 @@ regex_checks = {
     'Description': https_enforcings + http_url_shorteners + [
         (re.compile(r'\s*[*#][^ .]'),
          _("Invalid bulleted list")),
+        (re.compile(r'https://f-droid.org/[a-z][a-z](_[A-Za-z]{2,4})?/'),
+         _("Locale included in f-droid.org URL")),
         (re.compile(r'^\s'),
          _("Unnecessary leading space")),
         (re.compile(r'.*\s$'),
@@ -313,7 +315,7 @@ all_categories = set([
 def check_categories(app):
     for categ in app.Categories:
         if categ not in all_categories:
-            yield _("Category '%s' is not valid" % categ)
+            yield _("Categories '%s' is not valid" % categ)
 
 
 def check_duplicates(app):
@@ -473,6 +475,29 @@ def check_extlib_dir(apps):
         yield _("Unused extlib at %s") % os.path.join(dir_path, path)
 
 
+def check_app_field_types(app):
+    """Check the fields have valid data types"""
+
+    for field in app.keys():
+        v = app.get(field)
+        t = metadata.fieldtype(field)
+        if v is None:
+            continue
+        elif field == 'builds':
+            if not isinstance(v, list):
+                yield(_("{appid}: {field} must be a '{type}', but it is a '{fieldtype}'!")
+                      .format(appid=app.id, field=field,
+                              type='list', fieldtype=v.__class__.__name__))
+        elif t == metadata.TYPE_LIST and not isinstance(v, list):
+            yield(_("{appid}: {field} must be a '{type}', but it is a '{fieldtype}!'")
+                  .format(appid=app.id, field=field,
+                          type='list', fieldtype=v.__class__.__name__))
+        elif t == metadata.TYPE_STRING and not type(v) in (str, bool, dict):
+            yield(_("{appid}: {field} must be a '{type}', but it is a '{fieldtype}'!")
+                  .format(appid=app.id, field=field,
+                          type='str', fieldtype=v.__class__.__name__))
+
+
 def check_for_unsupported_metadata_files(basedir=""):
     """Checks whether any non-metadata files are in metadata/"""
 
@@ -488,7 +513,13 @@ def check_for_unsupported_metadata_files(basedir=""):
             if not exists:
                 print(_('"%s/" has no matching metadata file!') % f)
                 return_value = True
-        elif not os.path.splitext(f)[1][1:] in formats:
+        elif os.path.splitext(f)[1][1:] in formats:
+            packageName = os.path.splitext(os.path.basename(f))[0]
+            if not common.is_valid_package_name(packageName):
+                print('"' + packageName + '" is an invalid package name!\n'
+                      + 'https://developer.android.com/studio/build/application-id')
+                return_value = True
+        else:
             print('"' + f.replace(basedir, '')
                   + '" is not a supported file format: (' + ','.join(formats) + ')')
             return_value = True
@@ -496,6 +527,35 @@ def check_for_unsupported_metadata_files(basedir=""):
     return return_value
 
 
+def check_current_version_code(app):
+    """Check that the CurrentVersionCode is currently available"""
+
+    archive_policy = app.get('ArchivePolicy')
+    if archive_policy and archive_policy.split()[0] == "0":
+        return
+    cv = app.get('CurrentVersionCode')
+    if cv is not None and int(cv) == 0:
+        return
+
+    builds = app.get('builds')
+    active_builds = 0
+    min_versionCode = None
+    if builds:
+        for build in builds:
+            vc = int(build['versionCode'])
+            if min_versionCode is None or min_versionCode > vc:
+                min_versionCode = vc
+            if not build.get('disable'):
+                active_builds += 1
+            if cv == build['versionCode']:
+                break
+    if active_builds == 0:
+        return  # all builds are disabled
+    if cv is not None and int(cv) < min_versionCode:
+        yield(_('CurrentVersionCode {cv} is less than oldest build entry {versionCode}')
+              .format(cv=cv, versionCode=min_versionCode))
+
+
 def main():
 
     global config, options
@@ -532,6 +592,7 @@ def main():
             continue
 
         app_check_funcs = [
+            check_app_field_types,
             check_regexes,
             check_update_check_data_url,
             check_vercode_operation,
@@ -549,6 +610,7 @@ def main():
             check_files_dir,
             check_format,
             check_license_tag,
+            check_current_version_code,
         ]
 
         for check_func in app_check_funcs:
